# Part of flinks
# (C) Martin Bays 2008
# Released under the terms of the GPLv3

import os,sys
from string import *

from StringIO import StringIO

import curses
import curses.ascii as ascii

from Browser import Browser

defaultConfig = """# options

set wpm 500
set initialUrl http://en.wikipedia.org
set showPoint True
set blankBetween True
set linkDisplayNum 4
set linkDisplayEmphNum 1
set trickyDisplayNum 3
set maxCached 5
set paraBlankSpace 2
set sentenceBlankSpace 1

# keybindings

bind > changespeed 5
bind k changespeed 5
bind K changespeed 25
bind Up changespeed 25
bind < changespeed -5
bind j changespeed -5
bind J changespeed -25
bind Down changespeed -25

bind l seekSentence 1
bind L seekParagraph 1
bind Right seekSentence 1
bind h seekSentence -1
bind H seekParagraph -1
bind Left seekSentence -1
bind o seekWord 20
bind O seekWord 100
bind NPage seekWord 100
bind i seekWord -20
bind I seekWord -100
bind PPage seekWord -100
bind . seekWord 1
bind , seekWord -1
bind ] seekLink 1
bind [ seekLink -1

bind g go
bind G goCurrent

bind / search 1
bind ? search -1
bind n searchAgain 1
bind N searchAgain -1

bind b history -1
bind Backspace history -1
bind u history 1

bind q quit

bind ^L refresh

bind 1 followLink 1
bind 2 followLink 2
bind 3 followLink 3
bind 4 followLink 4
bind 5 followLink 5
bind 6 followLink 6
bind 7 followLink 7
bind 8 followLink 8
bind 9 followLink 9

bind 0 count

bind m mark
bind ' goMark

bind space pause
"""

class Config:
    optionsInfo = {
		# <option name> : <type>
		'wpm' : int,
		'initialUrl' : str,
		'showPoint' : bool,
		'blankBetween' : bool,
		'linkDisplayNum' : int,
		'linkDisplayEmphNum' : int,
		'trickyDisplayNum' : int,
		'maxCached' : int,
		'paraBlankSpace' : int,
		'sentenceBlankSpace' : int
	    }

    def __init__(self):
	self.options = {}
	self.keyBindings = {}
	self.loadErrors = []

	self.load(StringIO(defaultConfig), "[Default Config]")

	filename = os.getenv('HOME')+'/.flinksrc'
	try:
	    file = open(filename, 'r')
	    self.load(file, filename)
	except IOError:
	    # write default config
	    open(filename, 'w').write(defaultConfig)

    def load(self, file, filename):
	lineNumber = 0

	def addError(details):
	    self.loadErrors.append("While parsing %s, line %d : %s" %
		    (filename, lineNumber, details))

	for line in file:
	    lineNumber += 1

	    words = split(line)

	    if words == [] or words[0][0] == '#':
		continue

	    command = words[0]

	    if command == "set":
		if (len(words) < 3):
		    addError("usage: set OPTION VALUE")
		    continue

		for option in self.optionsInfo.keys():
		    if words[1].lower() == option.lower():
			optionType = self.optionsInfo[option]
			if optionType == int:
			    try:
				self.options[option] = int(words[2])
			    except ValueError:
				addError("expected integer value")
			elif optionType == bool:
			    self.options[option] = (words[2].lower() == 'true')
			elif optionType == str:
			    self.options[option] = words[2]
			break

	    elif command == "bind":
		if (len(words) < 2):
		    addError("usage: bind KEY [FUNCTION [ARGS...]]")
		    continue

		key = None

		keyName = words[1]

		if len(keyName) == 1:
		    # A
		    key = ord(keyName)
		elif len(keyName) == 2 and keyName[0] == '^':
		    # ^A
		    key = ord(ascii.ctrl(keyName[1]))
		elif len(keyName) == 3 and keyName[1] == '-' and keyName[0] == 'C':
		    # C-A
		    key = ord(ascii.ctrl(keyName[2]))
		elif len(keyName) == 3 and keyName[1] == '-' and keyName[0] == 'M':
		    # M-A
		    key = ord(ascii.alt(keyName[2]))
		elif keyName.lower() == "space":
		    # space (special case)
		    key = ord(' ')
		else:
		    # down
		    try: 
			key = getattr(curses, 'KEY_'+keyName.upper())
		    except AttributeError:
			pass

		if key is None:
		    addError("bad key description \"%s\"" % (words[1]))
		    continue

		if len(words) == 2:
		    # clear binding
		    if self.keyBindings.has_key(key):
			del self.keyBindings[key]
		else:
		    # add binding
		    method = None
		    for possMethod in Browser.bindableMethods.keys():
			if words[2].lower() == possMethod.lower():
			    method = possMethod
		    if method is None:
			addError("unknown function \"%s\"" % words[2]) 
			continue

		    minArgs = Browser.bindableMethods[method][0]
		    maxArgs = Browser.bindableMethods[method][1]

		    if len(words) - 3 < minArgs or len(words) - 3 > maxArgs:
			addError("\"%s\" requires %d-%d arguments" %
				(method, minArgs, maxArgs)) 
			continue

		    args = []
		    for arg in words[3:]:
			try:
			    args.append(int(arg))
			except ValueError:
			    addError("expected integer argument")
			    continue

		    self.keyBindings[key] = [method, args]

	    else:
		addError("unknown command \"%s\"" % words[0])
